/**
 * 
 */
package hamsam.protocol.insecureSample;

import hamsam.api.Conference;
import hamsam.api.Message;
import hamsam.api.MessageComponent;
import hamsam.api.Response;
import hamsam.api.SmileyComponent;

import java.io.IOException;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Set;

/**
 * 
 * There is one InsecureServerLogic instance per client connection.  
 * Static variables are used to store information like conference 
 * attendees and buddy lists between connections.
 */
public class InsecureServerLogic {

	/** 
	 * The username of the buddy associated with this Protocol instance.
	 */
	private String username = null;
	private String status = null;
	private Set buddyList = null;
	private Set ignoreList = null;
	
	/**
	 * A map from String (username) to InsecureServerLogic
	 */
	protected static Hashtable insecureProtocolServers = new Hashtable();	// Hashtable<String, InsecureProtoocolServer>
	
	// These two hashtables exist because their contents persist across connections.
	private static Hashtable buddyLists = new Hashtable();  			// Hashtable<String, Set>
	private static Hashtable ignoreLists = new Hashtable(); 			// Hashtable<String, Set>
	
	/**
	 * A map from conference name to the name of the buddy that initiated the conference.  This name is only used so that we can 
	 * generate appropriate CONFERENCE_INVITATION_ACCEPTED and CONFERENCE_INVITATION_REJECTED messages. 
	 */
	private static Hashtable conferenceOwners = new Hashtable(); // Hashtable<String, String>
	/**
	 * A map from conference name to set of conference attendees.  In java 1.5 generics syntax, this is a Hashtable<String, Set<String>>
	 */ 
	private static Hashtable conferenceAttendeeLists = new Hashtable(); // Hashtable<String, Set>

	/**
	 * A map from conference name to set of users that have been invited to the conference, but 
	 * that decided not to attend. In java 1.5 generics syntax, this is a Hashtable<String, Set<String>>. 
	 */
	private static Hashtable conferenceDeclinedLists = new Hashtable(); // Hashtable<String, Set<String>>

	/**
	 * A map from conference name to set of users that have been invited to the conference, but 
	 * that decided not to attend. In java 1.5 generics syntax, this is a Hashtable<String, Set<String>>. 
	 */
	private static Hashtable conferenceLeaverLists = new Hashtable(); // Hashtable<String, Set<String>>

	/**
	 * A map from conference name to set of users that have been invited to the conference but that 
	 * have not yet responded to the invitation.  In java 1.5 generics syntax, this is a 
	 * Hashtable<String, Set<String>>
	 *
	 * TODO Do we want to include this in the insecure example?  It's a security feature, isn't it?
	 */ 
	private static Hashtable conferenceInviteeLists = new Hashtable();  // Hashtable<String, Set>

	
	private InsecureNetworking wireFormat = null;
	
	/**
	 * @param userName the user name of the buddy associated with this Protocol instance.
	 * The Buddy's client will treat this Protocol instance like a connection handle. 
	 */
	public InsecureServerLogic(String username) {
		super();
		this.username = username;
	}

	public void changeStatus(String status) throws IllegalArgumentException,
			IllegalStateException {
		this.status = status;
		
		Iterator it = this.buddyList.iterator();
		while(it.hasNext()) { 
			String buddyName = (String)it.next();
			InsecureServerLogic buddyHandle = 
					(InsecureServerLogic)insecureProtocolServers.get(buddyName);
			
			// TODO Locking?!??
			buddyHandle.wireFormat.sendMessageToClient(InsecureNetworking.BUDDY_STATUS_CHANGED, 
													   new Serializable[] {this.username, 
													   				 this.status});
		}
	}

	public boolean connect(InsecureNetworking wireFormat, String username, String password)
			throws IllegalStateException {

		wireFormat.sendMessageToClient(InsecureNetworking.CONNECTING, new Serializable[0]);
		
		if(!username.equals(password)) { 
			wireFormat.sendMessageToClient(InsecureNetworking.CONNECT_FAILED, new Serializable[] { "Incorrect password.  Your password is the same string as your username" });
			return false;
		} else if (null != insecureProtocolServers.get(username)) { 
			// Kick the other client off the network.
			InsecureServerLogic stale = (InsecureServerLogic)insecureProtocolServers.get(username);
			stale.disconnect();
			// This would also work...
//			wireFormat.sendMessageToClient(InsecureNetworking.CONNECT_FAILED, new Serializable[] { "You're already online!" } );
//			return false;

		}
			
		this.wireFormat = wireFormat;
		insecureProtocolServers.put(username, this);
		this.username = username;
		this.status   = getSupportedStatusMessages()[0];  // "Connected"
		
		// Register a new buddy list and/or ignore list if necessary, and send a buddy list to the client.
		this.buddyList = (Set)buddyLists.get(username);
		if(this.buddyList == null) {
		        this.buddyList = new HashSet();
		        buddyLists.put(username, buddyList);
		}
		this.ignoreList = (Set)ignoreLists.get(username);
		if(this.ignoreList == null) {
			this.ignoreList = new HashSet();
			this.ignoreList.add("Gonzo");
			ignoreLists.put(username, ignoreList);
		}
		
		wireFormat.sendMessageToClient(InsecureNetworking.BUDDY_LIST_RECEIVED, 
								       new Serializable[] { buddyList.toArray(new String[0]) });
		wireFormat.sendMessageToClient(InsecureNetworking.IGNORE_LIST_RECEIVED,
									   new Serializable[] { ignoreList.toArray(new String[0]) });
		wireFormat.sendMessageToClient(InsecureNetworking.CONNECTED, new Serializable[] { username });
		
		return true;
	}

	public void disconnect() throws IllegalStateException {
		
		insecureProtocolServers.remove(username); // This buddy is no longer online, so remove this from the insecureProtocolServers map.
		
		wireFormat.sendMessageToClient(InsecureNetworking.DISCONNECTED, new Serializable[] {});
		
	}

	public String getProtocolName() {
		return "Insecure CS 161 sample protocol";
	}

	public boolean isBuddyAddRequestSupported() { 	return true;	}
	public boolean isBuddyNameAliasSupported() {	return false;	}
	public boolean isBuddyGroupSupported() {		return false;	}
	public boolean isIgnoreSupported() {
		// TODO RCS Should the students be expected to implement this securely?  (How can a p2p system support this without letting the ignored buddy know that he's being ignored?)
		return true;
	}
	public boolean isOfflineMessageSupported() {	return false;	}
	public boolean isTypingNotifySupported() {		return false;	}
	public boolean isConferenceSupported() {		return true;	}
	public boolean isMailNotifySupported() {		return false;	}
	public boolean isInvisibleSupported() {			return false;	}

	public void startConference(String conferenceName, String message, String[] invitees)
			throws UnsupportedOperationException, IllegalStateException {

		HashSet attendeeSet = new HashSet();
		attendeeSet.add(username);
		conferenceAttendeeLists.put(conferenceName, attendeeSet);
		conferenceDeclinedLists.put(conferenceName, new HashSet());
		HashSet inviteeSet = new HashSet();
		for(int i = 0; i < invitees.length; i++) {
			if(!invitees[i].equals(username)) { 
				inviteeSet.add(invitees[i]);
			}
		}

		
		conferenceInviteeLists.put(conferenceName, inviteeSet);
		conferenceLeaverLists.put(conferenceName, new HashSet());
		conferenceOwners.put(conferenceName, username);

		for(int i = 0; i < invitees.length; i++) { 
			try { 
				if(!invitees[i].equals(username)) {
					System.out.println("Sending invitation to conference " + conferenceName + " from " + username + " for " + invitees[i] + " message: " + message);
					InsecureServerLogic attendee = 
						(InsecureServerLogic)insecureProtocolServers.get(invitees[i]);
					attendee.wireFormat.sendMessageToClient(InsecureNetworking.CONFERENCE_INVITATION_RECEIVED, 
															new Serializable[] {conferenceName,  username, message, invitees});
				}
			} catch(NullPointerException e) { 
				wireFormat.sendMessageToClient(InsecureNetworking.CONFERENCE_INVITATION_DECLINED,
											   new Serializable[] { conferenceName, 
											   			      invitees[i], "User " + invitees[i] + " doesn't exist."} );	
			}
		}
	}

	public void conferenceInvitationResponse(String conferenceName, Response response) { 
		Set conferenceAttendees = (Set)(conferenceAttendeeLists.get(conferenceName));
		if(conferenceAttendees == null) { 
			wireFormat.sendMessageToClient(InsecureNetworking.CONFERENCE_CLOSED, new Serializable[] { conferenceName });
		}
		Set conferenceDecliners = (Set)(conferenceDeclinedLists.get(conferenceName));
		Set conferenceLeavers   = (Set)(conferenceLeaverLists.get(conferenceName));
		Set conferenceInvitees  = (Set)(conferenceInviteeLists.get(conferenceName));
		conferenceInvitees.remove(username);

		if(response.isAccepted()) { 
			// Tell all of the other members that username has accepted the invitation, and joined the conference.
			// and tell username who has joined the conference, and who has declined the invitation.
			//
			// Note that there is some ambuguity here.  CONFERENCE_INVITATION_DECLINED and CONFERENCE_PARTICIPANT_LEFT 
			// clearly have different semantics; Buddies that never decline invitations never join; therefore they can 
			// never leave.  Similarly, Buddies that accept invitations may only leave, and not decline.   
			//
			// The ambiguity comes in when we think about the difference between JOINED and ACCEPTED.  This code uses 
			// the following interpretation:
			//
			// CONFERENCE_INVITATION_ACCEPTED is a real-time message, and is sent when the invitation is accepted.  
			// CONFERENCE_PARTICIPANT_JOINED reflects information about the current state of the conference, and is sent
			// both immediately after the invitation is accepted, and when client state needs to be updated. 

			Iterator attendees = conferenceAttendees.iterator();
			while(attendees.hasNext()) { 
				String member = (String) attendees.next();
				InsecureServerLogic attendee = 
					(InsecureServerLogic)insecureProtocolServers.get(member);
				attendee.wireFormat.sendMessageToClient(InsecureNetworking.CONFERENCE_INVITATION_ACCEPTED,
														new Serializable [] { conferenceName, username });
				attendee.wireFormat.sendMessageToClient(InsecureNetworking.CONFERENCE_PARTICIPANT_JOINED, 
														new Serializable [] { conferenceName, username });
				wireFormat.sendMessageToClient(InsecureNetworking.CONFERENCE_PARTICIPANT_JOINED,
											   new Serializable [] { conferenceName, member});
			}
			Iterator decliners = conferenceDecliners.iterator();
			while(decliners.hasNext()) {
				String member = (String) decliners.next();
				wireFormat.sendMessageToClient(InsecureNetworking.CONFERENCE_INVITATION_DECLINED,
											   new Serializable [] { conferenceName, member, member + " did not want to be in the confrence." });
			}
			Iterator leavers = conferenceLeavers.iterator();
			while(leavers.hasNext()) { 
				String member = (String) leavers.next();
				wireFormat.sendMessageToClient(InsecureNetworking.CONFERENCE_PARTICIPANT_LEFT, new Serializable [] { conferenceName, member });
			}
			// Add username to the conference.
			conferenceAttendees.add(username);
		} else { 
			Iterator attendees = conferenceAttendees.iterator();
			while(attendees.hasNext()) {
				String member = (String)attendees.next();
				InsecureServerLogic attendee = 
					(InsecureServerLogic)insecureProtocolServers.get(member);
				attendee.wireFormat.sendMessageToClient(InsecureNetworking.CONFERENCE_INVITATION_DECLINED, 
														new Serializable [] { conferenceName, username, username + " does not want to be in the confrence." });
			}
			conferenceDecliners.add(username);
		}
		
        tearDownConferenceIfNecessary(conferenceName);
	}

	public void quitConference(String conferenceName)
			throws UnsupportedOperationException, IllegalStateException {
		Set conferenceMembers = (Set) conferenceAttendeeLists.get(conferenceName);
		if(conferenceMembers == null) { 
			return; // conference is already closed. 
		}
		conferenceMembers.remove(username);
		Set conferenceLeavers = (Set) conferenceLeaverLists.get(conferenceName);
		conferenceLeavers.add(username);
		Iterator members = conferenceMembers.iterator();
		while(members.hasNext()) { 
			String member = (String)members.next();
			InsecureServerLogic attendee = 
				(InsecureServerLogic)insecureProtocolServers.get(member);
			attendee.wireFormat.sendMessageToClient(InsecureNetworking.CONFERENCE_PARTICIPANT_LEFT, 
													new Serializable [] { conferenceName, username });
		}
		
		tearDownConferenceIfNecessary(conferenceName);
	}

	public void tearDownConferenceIfNecessary(String conferenceName) { 
		Set conferenceMembers = (Set)conferenceAttendeeLists.get(conferenceName);
		Set conferenceInvitees = (Set)conferenceInviteeLists.get(conferenceName);
		System.out.println(conferenceMembers.size() + " buddies in conference.  Wating for RSVP's from " + conferenceInvitees.size() + " other buddies.");
		if(conferenceMembers.size() + conferenceInvitees.size() < 2) {
			// Tear down conference.
			Iterator it = conferenceMembers.iterator();
			while(it.hasNext()) { 
				String member = (String)it.next();
				InsecureServerLogic attendee = 
					(InsecureServerLogic)insecureProtocolServers.get(member);
				attendee.wireFormat.sendMessageToClient(InsecureNetworking.CONFERENCE_CLOSED, new Serializable[] { conferenceName });
			}
			conferenceAttendeeLists.remove(conferenceName);
			conferenceDeclinedLists.remove(conferenceName);
			conferenceInviteeLists.remove(conferenceName);
			conferenceLeaverLists.remove(conferenceName);
			conferenceOwners.remove(conferenceName);
		} 
	}
	
	public void sendConferenceMessage(String conferenceName, Message message)
			throws UnsupportedOperationException, IllegalStateException {
		System.out.println("Got a conference message from " + username + " for conference " + conferenceName);
		Set conferenceMembers = (Set) conferenceAttendeeLists.get(conferenceName);
		Iterator it = conferenceMembers.iterator();
		while(it.hasNext()) { 
			String member = (String)it.next();
			if(!username.equals(member)) { 
				InsecureServerLogic attendee = 
					(InsecureServerLogic)insecureProtocolServers.get(member);
				try { 
					attendee.wireFormat.sendMessageToClient(InsecureNetworking.CONFERENCE_MESSAGE_RECEIVED, 
							new Serializable[] { conferenceName, 
							username,
							message });
				} catch (Exception e) { 
					//TODO What should we do here?  An attendee has disconnected, but the conference data structures don't know!
					e.printStackTrace();
				}
			}
		}
	}

	public void addToBuddyList(String buddy, String message) throws IllegalArgumentException,
			IllegalStateException {
		if(buddyList.contains(buddy)) { 
			wireFormat.sendMessageToClient(InsecureNetworking.BUDDY_ADD_FAILED, new Serializable[] { buddy, buddy + " is already on your buddy list." });
		} else { 
			InsecureServerLogic recipient = (InsecureServerLogic) insecureProtocolServers.get(buddy);
			//TODO Hole in hamsam API.  Need to hard code add request string?
			if(recipient == null || ! recipient.wireFormat.sendMessageToClient(InsecureNetworking.BUDDY_ADD_REQUEST, new Serializable[] {username, buddy, message})) { 
				wireFormat.sendMessageToClient(InsecureNetworking.BUDDY_ADD_FAILED, new Serializable[] { buddy, buddy + " is not online." });
			}
		}
	}

	public void deleteFromBuddyList(String buddy)
			throws IllegalArgumentException, IllegalStateException {
		if(this.buddyList.remove(buddy)) { 
			wireFormat.sendMessageToClient(InsecureNetworking.BUDDY_DELETED, new Serializable[] { buddy });
		} else { 
			wireFormat.sendMessageToClient(InsecureNetworking.BUDDY_DELETE_FAILED, new Serializable[] { buddy, buddy + " is not on your buddy list!"});
		}
	}

	public void sendInstantMessage(String buddy, Message message)
			throws IllegalStateException {
		InsecureServerLogic recipient = (InsecureServerLogic) insecureProtocolServers.get(buddy);
		if(recipient == null || ! recipient.wireFormat.sendMessageToClient(InsecureNetworking.INSTANT_MESSAGE_RECEIVED, 
												 new Serializable[] { username, message })) { 
			// TODO: We shouldn't silently drop the message!
		}
	}

	// Methods in this section are protocol specific
	
	public void addBuddyResponse(String buddy, Response response) {
		InsecureServerLogic recipient = (InsecureServerLogic) insecureProtocolServers.get(buddy);
		if(response.isAccepted()) { 
			recipient.buddyList.add(username); 
			recipient.wireFormat.sendMessageToClient(InsecureNetworking.BUDDY_ADDED, new Serializable[] { username, response.getMessage() });
		} else { 
			recipient.wireFormat.sendMessageToClient(InsecureNetworking.BUDDY_ADD_REJECTED, new Serializable[] { username, response.getMessage() });
		}
	}	
	// More hamsam inspired methods.

	public String[] getSupportedStatusMessages() {
		return new String[] { "Online", "Offline" };
	}

	// --- Our protocol doesn't support any of the calls defined below this line.
	
	public void ignoreBuddy(String buddy) throws UnsupportedOperationException,
			IllegalStateException {
		throw new UnsupportedOperationException("Ignore buddy notifications are not supported by this protocol");
	}

	public void unignoreBuddy(String buddy)
			throws UnsupportedOperationException, IllegalStateException {
		throw new UnsupportedOperationException("Ignore buddy notifications are not supported by this protocol");
	}

	public void typingStarted(String buddy)
			throws UnsupportedOperationException, IllegalStateException {
		throw new java.lang.UnsupportedOperationException("Typing started notifications are not supported by this protocol");
	}

	public void typingStopped(String buddy)
			throws UnsupportedOperationException, IllegalStateException {
		throw new java.lang.UnsupportedOperationException("Typing stopped notifications are not supported by this protocol");
	}

	public void changeBuddyAlias(String buddy, String alias)
			throws UnsupportedOperationException {
		throw new java.lang.UnsupportedOperationException("Buddy aliases are not supported by this protocol");
	}

}
